/*
    SPDX-FileCopyrightText: 2023 David Edmundson <davidedmundson@kde.org>
    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#pragma once

#include <Qt>

namespace PlasmaQuickPrivate
{
Qt::Edge oppositeEdge(Qt::Edge edge);
}
